// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO20/16AO20_Linux_2.x.x.x_GSC_DN/samples/sbtest/reg_read.c $
// $Rev: 52761 $
// $Date: 2023-04-05 15:08:50 -0500 (Wed, 05 Apr 2023) $

// 16AO20: Sample Application: source file

#include "main.h"



/******************************************************************************
*
*	Function:	reg_read_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service AO20_IOCTL_REG_READ.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int reg_read_test(int fd)
{
	static const struct
	{
		u32	reg;
		u32	value;
	} list[]	=
	{
		// reg					value
		{ AO20_GSC_BCR,			0		},
		{ AO20_GSC_CSR,			0		},
		{ AO20_GSC_SRR,			0		},
		{ AO20_GSC_BOR,			0		},
		{ AO20_GSC_AVR,			0		},
		{ AO20_GSC_FOR,			0		},
		{ AO20_GSC_AVR,			0		},
		{ AO20_GSC_ODBR,		0		},
		{ AO20_GSC_ACR,			0		},

		{ GSC_PCI_9080_VIDR,	0x10B5	},
		{ GSC_PCI_9080_DIDR,	0x9080	},
		{ GSC_PCI_9080_SVID,	0x10B5	},
		{ GSC_PCI_9080_SID,		0x3102	},

		{ GSC_PLX_9080_VIDR,	0x10B5	},
		{ GSC_PLX_9080_DIDR,	0x9080	}
	};

	int			errs	= 0;
	int			i;
	gsc_reg_t	parm;
	int			ret;

	gsc_label("AO20_IOCTL_REG_READ");

	for (i = 0; i < SIZEOF_ARRAY(list); i++)
	{
		parm.reg	= list[i].reg;
		parm.value	= 0xDEADBEEF;
		parm.mask	= 0xBEEFDEAD;
		ret			= ao20_ioctl(fd, AO20_IOCTL_REG_READ, (void*) &parm);

		if (ret)
		{
			errs	= 1;
			printf(	"FAIL <---  (%d. i %d, ao20_ioctl() failure, error %d)\n",
					__LINE__,
					i,
					ret);
			break;
		}

		if (parm.value == 0xDEADBEEF)
		{
			errs	= 1;
			printf("FAIL <---  (%d. i %d, value not changed)\n", __LINE__, i);
			break;
		}

		if (parm.mask != 0xBEEFDEAD)
		{
			errs	= 1;
			printf("FAIL <---  (%d. i %d, mask changed)\n", __LINE__, i);
			break;
		}

		if ((list[i].value) && (parm.value != list[i].value))
		{
			errs	= 1;
			printf(	"FAIL <---  (%d. i %d, got 0x%lX, expected 0x%lX)\n",
					__LINE__,
					i,
					(long) parm.value,
					(long) list[i].value);
			break;
		}
	}

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


