// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO20/16AO20_Linux_2.x.x.x_GSC_DN/samples/sbtest/tx_io_over_frame.c $
// $Rev: 52761 $
// $Date: 2023-04-05 15:08:50 -0500 (Wed, 05 Apr 2023) $

// 16AO20: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AO20_IOCTL_TX_IO_OVER_FRAME,
			/* arg		*/	AO20_TX_IO_OVER_FRAME_IGNORE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AO20_IOCTL_TX_IO_OVER_FRAME,
			/* arg		*/	AO20_TX_IO_OVER_FRAME_CHECK,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int errs	= 0;

	errs	+= ao20_initialize(fd, -1, 0);

	errs	+= service_ioctl_set_get_list(fd, list);

	errs	+= service_ioctl_set_get_list(fd, list);

	errs	+= ao20_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	static const service_data_t	list_1[]	=
	{
		{					// Make the buffer closed.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AO20_IOCTL_BUFFER_MODE,
			/* arg		*/	AO20_BUFFER_MODE_CIRC,
			/* reg		*/	AO20_GSC_BOR,
			/* mask		*/	0x100,
			/* value	*/	0x100
		},
		{					// Cause a frame overflow.
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	AO20_GSC_ODBR,
			/* mask		*/	0,
			/* value	*/	0x0000
		},
		{					// Verify that there IS a data overflow.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AO20_GSC_BOR,
			/* mask		*/	0x20000,
			/* value	*/	0x20000
		},
		{					// Ignore the frame overflow.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AO20_IOCTL_TX_IO_OVER_FRAME,
			/* arg		*/	AO20_TX_IO_OVER_FRAME_IGNORE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Set a short second timeout.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AO20_IOCTL_TX_IO_TIMEOUT,
			/* arg		*/	3,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_2[]	=
	{
		{					// Check the overflow.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AO20_IOCTL_TX_IO_OVER_FRAME,
			/* arg		*/	AO20_TX_IO_OVER_FRAME_CHECK,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	s32		_1_4k;
	s32		_2_m1;
	s32		_3_to;
	u32		buf[1024]	= { 0 };
	int		errs		= 0;
	time_t	t1;
	time_t	t2;
	time_t	t3;
	time_t	t4;

	errs	+= ao20_initialize(fd, -1, 0);

	// Ignore the overflow.
	errs	+= service_ioctl_set_reg_list(fd, list_1);

	t1		= time(NULL);
	_1_4k	= ao20_write(fd, buf, sizeof(buf));
	t2		= time(NULL);

	// Check for the overflow.
	errs	+= service_ioctl_set_reg_list(fd, list_2);

	t3		= time(NULL);
	_2_m1	= ao20_write(fd, buf, sizeof(buf));
	_3_to	= _2_m1;
	t4		= time(NULL);

	if (errs)
	{
	}
	else if (_1_4k != 4096)
	{
		errs++;
		printf("FAIL <--- (Test 1 results are not 4096.)\n");
	}
	else if ((t2 - t1) > 1)
	{
		errs++;
		printf(	"FAIL <--- (The 1st ao20_write() took over 1 second.)\n");
	}
	else if (_2_m1 >= 0)
	{
		errs++;
		printf("FAIL <--- (Test 2 results are not -1.)\n");
	}
	else if (_3_to != -EIO)
	{
		errs++;
		printf("FAIL <--- (Test 3 results are not EIO.)\n");
	}
	else if ((t4 - t3) > 1)
	{
		errs++;
		printf("FAIL <--- (The 2nd ao20_write() took over 1 second.)\n");
	}

	errs	+= ao20_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	tx_io_over_frame_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service AO20_IOCTL_TX_IO_OVER_FRAME.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int tx_io_over_frame_test(int fd)
{
	int	errs	= 0;

	gsc_label("AO20_IOCTL_TX_IO_OVER_FRAME");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


