// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO20/16AO20_Linux_2.x.x.x_GSC_DN/utils/util_config_ao.c $
// $Rev: 52760 $
// $Date: 2023-04-05 15:06:22 -0500 (Wed, 05 Apr 2023) $

// 16AO20: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ao20_config_ao
*
*	Purpose:
*
*		Configure the given device using common defaults.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		fsamp	This is the desired Fsamp rate, of -1 to use the default.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int	ao20_config_ao(int fd, int index, int verbose, s32 fsamp)
{
	int	errs	= 0;
	s32	mask	= 0;
	s32	nclk;
	s32	nrate;
	s32	osc_src;

	if (verbose)
	{
		gsc_label_index("Output Configuration", index);
		printf("\n");
		gsc_label_level_inc();
	}

	errs	+= ao20_tx_io_mode			(fd, index, verbose, GSC_IO_MODE_PIO,				NULL);
	errs	+= ao20_tx_io_over_data		(fd, index, verbose, AO20_TX_IO_OVER_DATA_CHECK,	NULL);
	errs	+= ao20_tx_io_over_frame	(fd, index, verbose, AO20_TX_IO_OVER_FRAME_CHECK,	NULL);
	errs	+= ao20_tx_io_timeout		(fd, index, verbose, 30,							NULL);
	errs	+= ao20_initialize			(fd, index, verbose);
	errs	+= ao20_buffer_mode			(fd, index, verbose, AO20_BUFFER_MODE_OPEN,			NULL);
	errs	+= ao20_buffer_size			(fd, index, verbose, AO20_BUFFER_SIZE_256K,			NULL);
	errs	+= ao20_output_mode			(fd, index, verbose, AO20_OUTPUT_MODE_SIM,			NULL);
	errs	+= ao20_query				(fd, index, verbose, AO20_QUERY_RANGE,				NULL);
	errs	+= ao20_sample_mode			(fd, index, verbose, AO20_SAMPLE_MODE_CONT,			NULL);
	errs	+= ao20_burst_trig_src		(fd, index, verbose, AO20_BURST_TRIG_SRC_SW,		NULL);
	errs	+= ao20_query				(fd, index, verbose, AO20_QUERY_CHANNEL_MASK, &mask);
	errs	+= ao20_channel_sel			(fd, index, verbose, mask,							NULL);
	errs	+= ao20_data_format			(fd, index, verbose, AO20_DATA_FORMAT_OFF_BIN,		NULL);
	errs	+= ao20_ground_sense		(fd, index, verbose, AO20_GROUND_SENSE_DISABLE,		NULL);

	errs	+= ao20_fsamp_ao_compute	(fd, index, verbose, fsamp, &osc_src, &nclk, &nrate, NULL);
	errs	+= ao20_clock_osc_src		(fd, index, verbose, osc_src,						NULL);
	errs	+= ao20_clock_src			(fd, index, verbose, AO20_CLOCK_SRC_INT,			NULL);
	errs	+= ao20_nclk				(fd, index, verbose, nclk,							NULL);
	errs	+= ao20_nrate				(fd, index, verbose, nrate,							NULL);

	errs	+= ao20_ext_clk_in_pol		(fd, index, verbose, AO20_EXT_POL_NORMAL,			NULL);
	errs	+= ao20_ext_trig_in_pol		(fd, index, verbose, AO20_EXT_POL_NORMAL,			NULL);
	errs	+= ao20_ext_trig_out_pol	(fd, index, verbose, AO20_EXT_POL_NORMAL,			NULL);
	errs	+= ao20_xcvr_type			(fd, index, verbose, AO20_XCVR_TYPE_LVDS,			NULL);
	errs	+= ao20_clock_enable		(fd, index, verbose, AO20_CLOCK_ENABLE_YES,			NULL);
	errs	+= ao20_auto_calibrate		(fd, index, verbose);
	errs	+= ao20_buffer_clear		(fd, index, verbose);
	errs	+= ao20_buffer_over_data	(fd, index, verbose, AO20_BUFFER_OVER_DATA_CLR,		NULL);
	errs	+= ao20_buffer_over_frame	(fd, index, verbose, AO20_BUFFER_OVER_FRAME_CLR,	NULL);
	errs	+= ao20_buffer_status		(fd, index, verbose,								NULL);

	if (verbose)
		gsc_label_level_dec();

	return(errs);
}


