// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO20/16AO20_Linux_2.x.x.x_GSC_DN/utils/util_fref.c $
// $Rev: 52760 $
// $Date: 2023-04-05 15:06:22 -0500 (Wed, 05 Apr 2023) $

// 16AO20: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ao20_fref_compute
*
*	Purpose:
*
*		Configure the device, then capture data.
*
*	Arguments:
*
*		fd		The handle for the deviceto access.
*
*		index	The index of the device being accessed or -1 to ignore.
*
*		verbose	Work verbosely?
*
*		get		The computed Fref value is put here.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ao20_fref_compute(int fd, int index, int verbose, s32* get)
{
	int	errs	= 0;
	s32	nclk;
	s32	nclk_max;
	s32	nclk_min;
	s32	osc_src;
	s32	set;

	if (verbose)
		gsc_label_index("Fref", index);

	errs	+= ao20_query(fd, -1, 0, AO20_QUERY_NCLK_MAX, &nclk_max);
	errs	+= ao20_query(fd, -1, 0, AO20_QUERY_NCLK_MIN, &nclk_min);
	errs	+= ao20_clock_osc_src(fd, -1, 0, -1, &osc_src);
	errs	+= ao20_nclk(fd, -1, 0, -1, &nclk);

	switch (osc_src)
	{
		default:

			errs++;
			set	= 0;

			if (verbose)
			{
				printf(	"FAIL <---  (Oscillator Source error: %ld)\n",
						(long) osc_src);
			}

			break;

		case AO20_CLOCK_OSC_SRC_PRI:

			set	= 30000000L;

			if (verbose)
			{
				gsc_label_long_comma(set);
				printf("  (Source = Primary)\n");
			}

			break;

		case AO20_CLOCK_OSC_SRC_ALT:

			set	= (s32) (16000000.0 * (511.0 + nclk) / 511);

			if (verbose)
			{
				gsc_label_long_comma(set);
				printf("  (Source = Alternate, Nclk = %ld)\n", (long) nclk);
			}

			break;
	}

	if (get)
		get[0]	= set;

	return(errs);
}


